%macro cxa_avae_pull_orig_vstr_dim(
  sessionData=
 ,sessionDataForFullRecalc=
 ,loadDttmsForFullRecalc=
 ,outputTableToDelForFullRecalc=
 ,originationDim=
 ,outputOrigVstrDimData=
 ,unloadDttmsData= /*Used to calculate whether there is a CXT reprocessing or not.
                 If it is empty, no CXT reprocessing. Otherwise, fullRecalc
                 gets switched on.*/
 ,newLoadDttmsData=
 ,fullRecalc=N
);
 
  %global load_retvar;
  %let load_retvar=;
 
  /*Keep track of temp tables created so we can delete them at the end,*/
  %local tempTablesToDel;
  %let tempTablesToDel=;

  /*Create table of visitor originations for new sessions.
    It will only contain visitor_id where new_visitor_flag is set,
    unless this macro gets customized.*/
 
  /*Delete table if it already exists.*/
  %if %sysfunc(exist(&outputOrigVstrDimData)) %then %do;
    proc sql noprint;
    drop table &outputOrigVstrDimData;
    quit;
    %if %cxa_avae_check_err() %then %goto badEnd;
  %end;
 
  /*Are there any records in the unloads table? If so, switch on
    fullRecalc.*/
  %if %sysfunc(exist(&unloadDttmsData)) %then %do;
    %cxa_avae_count_nobs(&unloadDttmsData)
    %if %cxa_avae_check_err() or &cntObs=ERR %then %goto badEnd;
    %if &cntObs ne OK and &cntObs>0 %then %let fullRecalc=Y;
  %end;
 
  %if &fullRecalc=Y %then %do;
    %cxa_avae_msg(78,MPUT,NOTE,Execution,For this run of this job we are doing a full recalculation of the whole table.)
 
    /*For this to work, we have to delete the full target table in LASR,
      as well as its HDFS backups.*/
    %if %sysfunc(exist(&outputTableToDelForFullRecalc)) %then %do;
      proc sql noprint;
      drop table &outputTableToDelForFullRecalc;
      quit;
      %if %cxa_avae_check_err() %then %goto badEnd;
    %end;
    %local delBak;
    %let delBak=%cxa_avae_mk_bakup_fl_name(&outputTableToDelForFullRecalc,&_submart_event_no);
    %if %sysfunc(exist(&delBak)) %then %do;
      proc sql noprint;
      drop table &delBak;
      quit;
      %if %cxa_avae_check_err() %then %goto badEnd;
    %end;
  %end;
 
  %local useSessData;
  %if &fullRecalc=Y %then %let useSessData=&sessionDataForFullRecalc;
  %else %let useSessData=&sessionData;
 
  /*Get connection information*/
  %local connectStringNm cxawhSchemaNm;
  %let connectStringNm=&submart_dbms_libname.ConnectToString;
  %let cxawhSchemaNm=&&&submart_cxa_source_libname.SchemaName;
  
  /*Grab origination for flagged first-time visitors in his load,
    whether full reload or not.*/
  %local tempNewFlaggedVstrs;
  %let tempNewFlaggedVstrs=&submart_dbms_libname..avae_tempNewFlaggedVstrs;  
  %let tempTablesToDel=&tempTablesToDel &tempNewFlaggedVstrs;
  %if %sysfunc(exist(&tempNewFlaggedVstrs)) %then %do;
    proc sql noprint;
    drop table &tempNewFlaggedVstrs;
    quit;
    %if %cxa_avae_check_err() %then %goto badEnd;
  %end;
  proc sql noprint;
  create table &tempNewFlaggedVstrs
  as select distinct
     s.visitor_id as visitor_sk
    ,origination_type_nm
    ,origination_nm
    ,origination_placement_nm
    ,origination_creative_nm
    ,origination_tracking_cd
  from &useSessData s
  left join
       &originationDim o
  on o.origination_sk=s.origination_sk
  where
  %if &fullRecalc=Y %then %do;
      s.new_visitor_flg='1'
      and s.load_complete_dttm in
      (select load_dttm from &loadDttmsForFullRecalc)
  %end;
  %else %do;
    s.new_visitors=1
  %end;
  ;
  quit;
  %if %cxa_avae_check_err() %then %goto badEnd;

  /*Create temporary non-unique index for temp table - new flagged vstrs*/
  %local indexes indxCnt i;
  %let indexes=visitor_sk;
  %let indxCnt=%sysfunc(countw(&indexes));
  %do i=1 %to &indxCnt;
     %cxa_avae_index_dbms_table(&tempNewFlaggedVstrs,%scan(&indexes,&i,%str( )),unique=)
     %if %cxa_avae_check_err() or &index_retvar ne OK %then %goto badEnd;
  %end;

  %if &fullRecalc ne Y %then %do;
    /*Get list of load_dttm values already in CXA.
      Not needed for full-load.*/
    %local temploadedDttms;
    %let temploadedDttms=avaedbms.avae_temploadedDttms;
    %let tempTablesToDel=&tempTablesToDel &temploadedDttms;
    %if %sysfunc(exist(&temploadedDttms)) %then %do;
      proc sql noprint;
      drop table &temploadedDttms;
      quit;
      %if %cxa_avae_check_err() %then %goto badEnd;
    %end;
    proc sql noprint;
    create table &temploadedDttms
    as select load_dttm from &loadDttmsForFullRecalc
    where load_dttm not in 
     (select load_dttm 
      from &newLoadDttmsData);
    quit;
    %if %cxa_avae_check_err() %then %goto badEnd;
  
    /*Get all visitors already loaded. Not needed
      for full-load.*/
    %local tempexistingVisitors;
    %let tempexistingVisitors=avaedbms.avae_tempexistingVisitors;
    %let tempTablesToDel=&tempTablesToDel &tempexistingVisitors;
    %if %sysfunc(exist(&tempexistingVisitors)) %then %do;
      proc sql noprint;
      drop table &tempexistingVisitors;
      quit;
      %if %cxa_avae_check_err() %then %goto badEnd;
    %end;
    proc sql noprint;
    create table &tempexistingVisitors
    as select distinct visitor_id
    from &sessionDataForFullRecalc where
     load_complete_dttm in
      (select load_dttm 
  	 from &temploadedDttms);
    quit;
    %if %cxa_avae_check_err() %then %goto badEnd;
  %end;

  /*Create temporary non-unique index for temp table - existing vstrs*/
  %let indexes=visitor_id;
  %let indxCnt=%sysfunc(countw(&indexes));
  %do i=1 %to &indxCnt;
     %cxa_avae_index_dbms_table(&tempexistingVisitors,%scan(&indexes,&i,%str( )),unique=)
     %if %cxa_avae_check_err() or &index_retvar ne OK %then %goto badEnd;
  %end;
  
  /*Identify new visitors which didn't have a flag row and
    are not already loaded. For full load, do not
    need final WHERE clause.*/
  %local tempothNewVisitors;
  %let tempothNewVisitors=&submart_dbms_libname..avae_tempothNewVisitors;  
  %let tempTablesToDel=&tempTablesToDel &tempothNewVisitors;
  %if %sysfunc(exist(&tempothNewVisitors)) %then %do;
    proc sql noprint;
    drop table &tempothNewVisitors;
    quit;
    %if %cxa_avae_check_err() %then %goto badEnd;
  %end;
/* S1309345 - Changed query, removed not in */
  proc sql noprint;
  create table &tempothNewVisitors
  as select distinct s.visitor_id as visitor_sk
  from &sessionData as s 
  	left join &tempNewFlaggedVstrs as a
  	on s.visitor_id=a.visitor_sk 
  %if &fullRecalc ne Y %then %do;
     left join &tempexistingVisitors as b
     on s.visitor_id=b.visitor_id
  %end;
  where a.visitor_sk is null
  %if &fullRecalc ne Y %then %do;
     and b.visitor_id is null
  %end;
  ;
  quit;
  %if %cxa_avae_check_err() %then %goto badEnd;
  
  /*Now create final output table, joining flagged first time visitors
    with not flagged first time visitors (assigning "Not Available" to
    the dimension columns for the latter.)*/
  proc sql noprint;
  &&&connectStringNm;
  execute(
    create table &cxawhSchemaNm..%scan(&outputOrigVstrDimData,2,.) 
    as select
           visitor_sk
          ,origination_type_nm
          ,origination_nm
          ,origination_placement_nm
          ,origination_creative_nm
          ,origination_tracking_cd

    from &cxawhSchemaNm..%scan(&tempNewFlaggedVstrs,2,.)

    union all 

    select 
           visitor_sk
           ,'Not Available' as origination_type_nm 
           ,'Not Available' as origination_nm 
           ,'Not Available' as origination_placement_nm 
           ,'Not Available' as origination_creative_nm 
           ,'Not Available' as origination_tracking_cd 
    from &cxawhSchemaNm..%scan(&tempothNewVisitors,2,.)
    ) by %scan(&&&connectStringNm,3);
  quit;  
  %if %cxa_avae_check_err() %then %goto badEnd;

  /*Do deletes of temporary intermediate tables.*/
  %local cntDels i curDel;
  %let cntDels=%sysfunc(countw(&tempTablesToDel,%str( )));
  %do i=1 %to &cntDels;
    %let curDel=%scan(&tempTablesToDel,&i,%str( ));
    %if %sysfunc(exist(&curDel)) %then %do;
      proc sql noprint;
      drop table &curDel;
      quit;
    %end;
  %end;

  %goto macEnd;
   
  %badEnd:
  %let load_retvar=ERR;
  /*Do deletes of temporary intermediate tables.*/
  %local cntDels i curDel;
  %let cntDels=%sysfunc(countw(&tempTablesToDel,%str( )));
  %do i=1 %to &cntDels;
    %let curDel=%scan(tempTablesToDel,&i,%str( ));
    %if %sysfunc(exist(&curDel)) %then %do;
      proc sql noprint;
      drop table &curDel;
      quit;
    %end;
  %end;
  
  /*Note this will only cause the running signal to be deleted if this job if job-flow has not past point of no return.*/
  %cxa_avae_abort(HARD)
  %goto ultEnd;
 
  %macEnd:
  %let load_retvar=OK;
 
  %ultEnd:
%mend cxa_avae_pull_orig_vstr_dim;
